# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.asset import gapic_version as package_version

__version__ = package_version.__version__


from google.cloud.asset_v1.services.asset_service.async_client import (
    AssetServiceAsyncClient,
)
from google.cloud.asset_v1.services.asset_service.client import AssetServiceClient
from google.cloud.asset_v1.types.asset_service import (
    AnalyzeIamPolicyLongrunningMetadata,
    AnalyzeIamPolicyLongrunningRequest,
    AnalyzeIamPolicyLongrunningResponse,
    AnalyzeIamPolicyRequest,
    AnalyzeIamPolicyResponse,
    AnalyzeMoveRequest,
    AnalyzeMoveResponse,
    AnalyzeOrgPoliciesRequest,
    AnalyzeOrgPoliciesResponse,
    AnalyzeOrgPolicyGovernedAssetsRequest,
    AnalyzeOrgPolicyGovernedAssetsResponse,
    AnalyzeOrgPolicyGovernedContainersRequest,
    AnalyzeOrgPolicyGovernedContainersResponse,
    AnalyzerOrgPolicy,
    AnalyzerOrgPolicyConstraint,
    BatchGetAssetsHistoryRequest,
    BatchGetAssetsHistoryResponse,
    BatchGetEffectiveIamPoliciesRequest,
    BatchGetEffectiveIamPoliciesResponse,
    BigQueryDestination,
    ContentType,
    CreateFeedRequest,
    CreateSavedQueryRequest,
    DeleteFeedRequest,
    DeleteSavedQueryRequest,
    ExportAssetsRequest,
    ExportAssetsResponse,
    Feed,
    FeedOutputConfig,
    GcsDestination,
    GcsOutputResult,
    GetFeedRequest,
    GetSavedQueryRequest,
    IamPolicyAnalysisOutputConfig,
    IamPolicyAnalysisQuery,
    ListAssetsRequest,
    ListAssetsResponse,
    ListFeedsRequest,
    ListFeedsResponse,
    ListSavedQueriesRequest,
    ListSavedQueriesResponse,
    MoveAnalysis,
    MoveAnalysisResult,
    MoveImpact,
    OutputConfig,
    OutputResult,
    PartitionSpec,
    PubsubDestination,
    QueryAssetsOutputConfig,
    QueryAssetsRequest,
    QueryAssetsResponse,
    QueryResult,
    SavedQuery,
    SearchAllIamPoliciesRequest,
    SearchAllIamPoliciesResponse,
    SearchAllResourcesRequest,
    SearchAllResourcesResponse,
    TableFieldSchema,
    TableSchema,
    UpdateFeedRequest,
    UpdateSavedQueryRequest,
)
from google.cloud.asset_v1.types.assets import (
    Asset,
    AttachedResource,
    ConditionEvaluation,
    EffectiveTagDetails,
    IamPolicyAnalysisResult,
    IamPolicyAnalysisState,
    IamPolicySearchResult,
    RelatedAsset,
    RelatedAssets,
    RelatedResource,
    RelatedResources,
    RelationshipAttributes,
    Resource,
    ResourceSearchResult,
    Tag,
    TemporalAsset,
    TimeWindow,
    VersionedResource,
)

__all__ = (
    "AssetServiceClient",
    "AssetServiceAsyncClient",
    "AnalyzeIamPolicyLongrunningMetadata",
    "AnalyzeIamPolicyLongrunningRequest",
    "AnalyzeIamPolicyLongrunningResponse",
    "AnalyzeIamPolicyRequest",
    "AnalyzeIamPolicyResponse",
    "AnalyzeMoveRequest",
    "AnalyzeMoveResponse",
    "AnalyzeOrgPoliciesRequest",
    "AnalyzeOrgPoliciesResponse",
    "AnalyzeOrgPolicyGovernedAssetsRequest",
    "AnalyzeOrgPolicyGovernedAssetsResponse",
    "AnalyzeOrgPolicyGovernedContainersRequest",
    "AnalyzeOrgPolicyGovernedContainersResponse",
    "AnalyzerOrgPolicy",
    "AnalyzerOrgPolicyConstraint",
    "BatchGetAssetsHistoryRequest",
    "BatchGetAssetsHistoryResponse",
    "BatchGetEffectiveIamPoliciesRequest",
    "BatchGetEffectiveIamPoliciesResponse",
    "BigQueryDestination",
    "CreateFeedRequest",
    "CreateSavedQueryRequest",
    "DeleteFeedRequest",
    "DeleteSavedQueryRequest",
    "ExportAssetsRequest",
    "ExportAssetsResponse",
    "Feed",
    "FeedOutputConfig",
    "GcsDestination",
    "GcsOutputResult",
    "GetFeedRequest",
    "GetSavedQueryRequest",
    "IamPolicyAnalysisOutputConfig",
    "IamPolicyAnalysisQuery",
    "ListAssetsRequest",
    "ListAssetsResponse",
    "ListFeedsRequest",
    "ListFeedsResponse",
    "ListSavedQueriesRequest",
    "ListSavedQueriesResponse",
    "MoveAnalysis",
    "MoveAnalysisResult",
    "MoveImpact",
    "OutputConfig",
    "OutputResult",
    "PartitionSpec",
    "PubsubDestination",
    "QueryAssetsOutputConfig",
    "QueryAssetsRequest",
    "QueryAssetsResponse",
    "QueryResult",
    "SavedQuery",
    "SearchAllIamPoliciesRequest",
    "SearchAllIamPoliciesResponse",
    "SearchAllResourcesRequest",
    "SearchAllResourcesResponse",
    "TableFieldSchema",
    "TableSchema",
    "UpdateFeedRequest",
    "UpdateSavedQueryRequest",
    "ContentType",
    "Asset",
    "AttachedResource",
    "ConditionEvaluation",
    "EffectiveTagDetails",
    "IamPolicyAnalysisResult",
    "IamPolicyAnalysisState",
    "IamPolicySearchResult",
    "RelatedAsset",
    "RelatedAssets",
    "RelatedResource",
    "RelatedResources",
    "RelationshipAttributes",
    "Resource",
    "ResourceSearchResult",
    "Tag",
    "TemporalAsset",
    "TimeWindow",
    "VersionedResource",
)
